# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimMultipleList,
    HprimString,
    Segment,
)

logger = logging.getLogger("driverng.bioplex_2200")


class BioPlex2200SegmentH(HeaderSegment):
    sender_name = HprimString(5)
    processing_id = HprimString(12, default="P")
    message_date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class BioPlex2200SegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    middle_initial = HprimString((6, 3))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)

    orders = HprimForeign("O", "patient")


class BioPlex2200SegmentO(Segment):
    specimen_id = HprimString((3, 1))
    level = HprimString((3, 2))
    rack = HprimString((3, 3))
    tube_position = HprimString((3, 4))
    lot_number = HprimString(4)
    test_id = HprimString((5, 4))
    auto_dilution = HprimString((5, 5))
    manual_dilution = HprimString((5, 6))
    priority = HprimString(6)
    order_date = HprimDate(7)
    collection_date = HprimDate(8)
    action_code = HprimString(12)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class BioPlex2200SegmentR(Segment):
    test_id = HprimString((3, 4))
    result_type = HprimString((3, 5))
    value = HprimString(4)
    units = HprimString(5)
    abnormal_flags = HprimMultipleList(7)
    result_status = HprimString(8)
    start_date = HprimDate(12)
    date_completed = HprimDate(13)


class BioPlex2200SegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)


class BioPlex2200SegmentQ(Segment):
    barcode = HprimString((3, 2))
    test_id = HprimString((5, 4))
    request_code = HprimString(13)


class BioPlex2200SegmentL(Segment):
    termination_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "bioplex_2200", "label": "BioPlex 2200"}
    SEGMENT_CLASS_MAP = {
        "H": BioPlex2200SegmentH,
        "P": BioPlex2200SegmentP,
        "O": BioPlex2200SegmentO,
        "R": BioPlex2200SegmentR,
        "C": BioPlex2200SegmentC,
        "Q": BioPlex2200SegmentQ,
        "L": BioPlex2200SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "bioplex_sender_name", label="Code émetteur", default="MCA"
        )
    ]

    def __init__(self, **kwargs):
        self.bioplex_sender_name = kwargs.pop("bioplex_sender_name", "MCA")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = BioPlex2200SegmentH(type_segment="H", **kwargs)
        message.end = BioPlex2200SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(
            sender_name=self.bioplex_sender_name, message_date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["priority", "action"],
            action={"del": "C", worklist.ELSE: ""},
            priority={"S": "S", worklist.ELSE: "R"},
        ):
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                middle_initial=patient.title,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, priority, action in patient.samples:
                for order in sample.orders:
                    if order.dilution and ";" in order.dilution:
                        dilutions = order.dilution.split(";")
                        for dilution in dilutions:
                            seg_o = seg_p.orders.add(
                                specimen_id=sample.sample_id,
                                test_id=order.code,
                                priority=priority,
                                order_date=sample.request_date,
                                collection_date=sample.collection_date,
                                action_code=action,
                                manual_dilution="1",
                                auto_dilution=dilution,
                            )
                    else:
                        seg_o = seg_p.orders.add(
                            specimen_id=sample.sample_id,
                            test_id=order.code,
                            priority=priority,
                            order_date=sample.request_date,
                            collection_date=sample.collection_date,
                            action_code=action,
                        )
                        if order.dilution and ";" not in order.dilution:
                            manual_dilution = order.dilution.split(":")[0]
                            seg_o.manual_dilution = manual_dilution
                            auto_dilution = order.dilution.split(":")[1]
                            seg_o.auto_dilution = auto_dilution

        if message.patients:
            return message

        elif context["query"]:
            seg_p = message.patients.add()
            for seg_q in context["query"].queries:
                seg_p.orders.add(
                    specimen_id=seg_q.barcode, priority="R", report_type="Y"
                )
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.barcode)
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        sample_id="{}_{}".format(seg_o.specimen_id, seg_o.level),
                        control_lot=seg_o.lot_number,
                        type="control",
                    )
                else:
                    rack = ""
                    if seg_o.rack and seg_o.tube_position:
                        rack = "{}_{}".format(seg_o.rack, seg_o.tube_position)
                    elif seg_o.rack and not seg_o.tube_position:
                        rack = seg_o.rack
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_id, rack=rack, type="patient"
                    )
                comments = []
                for seg_c in seg_o.commentaires:
                    comments.append(seg_c.comment_text)
                for seg_r in seg_o.results:
                    flags = []
                    if seg_r.abnormal_flags:
                        for flag in seg_r.abnormal_flags:
                            flags.append(flag)
                    code = seg_r.test_id
                    if seg_r.result_type:
                        code = "{}_{}".format(seg_r.test_id, seg_r.result_type)
                    sample.results.add(
                        code=code,
                        value=seg_r.value,
                        date=seg_r.date_completed,
                        unit=seg_r.units,
                        comments=comments,
                        flags=flags,
                    )

        return worklist
